% drawSpaceBounds(sz,spaceDepthCm,fc,cc)
% 
% Draw the bounds around the space visible by the camera
%
% sz -- size of the image [ni nj]
% spaceDepthCm -- farthest distance "visible", in cm
% fc - vector of focal lengths in i and j directions.
%       that is, fc = [fc_i fc_j]
% cc - camera center vector
%       pixel location of the camera center [cc_i cc_j]
function drawSpaceBounds(sz,spaceDepthCm,fc,cc)
    [ii,jj] = ndgrid([1 sz(1)],[1 sz(2)]);
    ii([4 3]) = ii([3 4]); % to traverse the corners of the image
    jj([4 3]) = jj([3 4]); % to traverse the corners of the image
    v = reprojectPoint(ii(:)',jj(:)',spaceDepthCm,fc,cc);
    lines = [v,                   zeros(size(v));
             v(:,2:end), v(:,1),  v              ];
    plot3( [lines(3,:); lines(6,:)], ...% z
           [lines(2,:); lines(5,:)], ...% y
           [lines(1,:); lines(4,:)], ...% x
           'r-'); 
    
    xlabel('z (cm)'); % z
    ylabel('y (cm)'); % y
    zlabel('x (cm)'); % x

    % Because we use a left-handed coordinate system, 
    % And because we use the axis in a different order,
    % we need to reverse some of the axis
    set(gca,'xdir','normal') % z
    set(gca,'ydir','reverse')% y
    set(gca,'zdir','reverse')% x
    
    axis equal
end
