% plotSphere(x,y,z,r,hSphere,color)
%
% x,y,z -- center of sphere
% r -- radius of sphere
% hSphere -- handle of previously-plotted sphere
%    defaults to [] -- create new sphere
% color -- color to draw the sphere
%    defaults to 'r' -- red
%
% Plots in "rotated" space used by drawSpaceBounds where to plot,
% you use, e.g.
%    surf(Z,Y,X) instead of surf(X,Y,Z)
function [hSphere] = plotSphere(x,y,z,r,hSphere,color)

if nargin < 6
    color = 'r';
end

[X,Y,Z] = sphere;
X = r*X+x;
Y = r*Y+y;
Z = r*Z+z;
if nargin < 5 || isempty(hSphere) || ~ishandle(hSphere)
    hSphere = surf(Z,Y,X);
    set(hSphere,'EdgeColor','none')
    set(hSphere,'FaceColor',color)
    set(hSphere,'FaceLighting','phong')
    camlight('right')
    pause(0.1);
else
    set(hSphere,'XData',Z);
    set(hSphere,'YData',Y);
    set(hSphere,'ZData',X);
    pause(0.1);
end
end
