% The first time you run this script,
% Run demo.m, which sets up the variables needed by this script and
% runs this script.
%
% For continue running, change the outer for loop to a while-true loop.
% Then, after cancelling the loop, use
%  clear camera
% to stop capture and close the camera.

% Copyright (c) 2010, A. Bhargav Anand
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

% Yoder, Summer 2014
%  Modified to plot position in 3D space based on calibrated camera info
%  and size of found bounding-box
% 
% Yoder, Winter 2016-2017: 
%  Adapted to use webcam interface instead of image aquisition 
%  interface. (The webcam interface seems more stable and
%  is used by the calibration toolbox.)


figure(2);

%start the video aquisition here
hDiffIm = [];
hImage = [];
hImageAxis = [];
camera = setupCamera();

for indFrame = 1:200
    % Get the snapshot of the current frame
    I = snapshot(camera);
    
    % Now to track red objects in real time
    % we have to subtract the red component 
    % from the grayscale image to extract the red components in the image.
    diff_im = imsubtract(I(:,:,1), rgb2gray(I));
    %Use a median filter to filter out noise
    diff_im = medfilt2(diff_im, [3 3]);
    if isempty(hDiffIm) || ~ishandle(hDiffIm)
        figure(3);
        hDiffIm = imshow(diff_im);
        figure(3)
        setpixelposition(gcf, figure3Position)
    else
        set(hDiffIm,'Cdata',diff_im);
    end
    % Convert the resulting grayscale image into a binary image.
    diff_im = im2bw(diff_im,0.20);
    
    % Remove all those regions less than 300px
    diff_im = bwareaopen(diff_im,300);
    
    % Label all the connected components in the image.
    bw = bwlabel(diff_im, 8);
    
    % Here we do the image blob analysis.
    % We get a set of properties for each labeled region.
    stats = regionprops(diff_im, 'BoundingBox', 'Centroid');
    
    % Display the image
    if isempty(hImage) || ~ishandle(hImage)
        figure(1)
        disp('Setting up figure 1');
        hImageAxis = gca;
        hImage = imshow(I);
        setpixelposition(gcf,figure1Position)
        hold on
    else
        set(hImage,'CData',I);
        pause(0.1);
    end
    
    if isempty(hImageAxis) || ~ishandle(hImageAxis)
        figure(1)
        hImageAxis = gca;
    end
    
    %This is a loop to bound the red objects in a rectangular box.
    set(hImageAxis,'nextplot','add');
    children = get(hImageAxis,'children');
    children = children(children ~= hImage);
    delete(children);
    for object = 1:length(stats)
        bb = stats(object).BoundingBox;
        bc = stats(object).Centroid;
        rectangle('Parent',hImageAxis,'Position',bb,'EdgeColor','r','LineWidth',2);
        plot(hImageAxis,bc(1),bc(2), '-m+');
        hText = text(bc(1)+15,bc(2), strcat('X: ', num2str(round(bc(1))), '    Y: ', num2str(round(bc(2)))),'Parent',hImageAxis);
        set(hText, 'FontName', 'Arial', 'FontWeight', 'bold', 'FontSize', 12, 'Color', 'yellow');
    end
    hold off
    
    % Here we move the ball
    if length(stats) >= 1
        width = max(bb(3:4));
        z = ballDiameterCm/(width/mean(fc));

        xy = bb(1:2)+bb(3:4)/2;
        ij = fliplr(xy); % Put xy into the coordinate frame we use.
        clear xy
        v = reprojectPoint(ij(1),ij(2),z,fc,cc);
        
        hSphere = plotSphere(v(1),v(2),v(3),ballDiameterCm/2,hSphere);
    end    
    
end
% Both the loops end here.

clear camera
